@@ -1,34 +1,76 @@
 # Written by Owen Williams
 # see LICENSE for license information
-import os,sys,time
+import os,sys,time, pwd
 import string
 import fnmatch
-import gnomevfs
 import urllib
 import HTMLParser
 import string
 import locale
 import gettext
 
+from subprocess import Popen, PIPE, STDOUT
+
 locale.setlocale(locale.LC_ALL, '')
 gettext.install('penguintv', '/usr/share/locale')
 gettext.bindtextdomain('penguintv', '/usr/share/locale')
 gettext.textdomain('penguintv')
 _=gettext.gettext
 
+RUNNING_SUGAR = os.environ.has_key('SUGAR_PENGUINTV')
 
-
-VERSION="2.0.2"
+if RUNNING_SUGAR:
+	#I do this in case we're running in a python environment that has lucene
+	#and/or gconf but we want to pretend they aren't there
+	HAS_LUCENE = False
+	HAS_GCONF = False
+	HAS_GNOMEVFS = False
+	HAS_PYXML = False
+else:
+	try:
+		import PyLucene
+		HAS_LUCENE = True
+	except:
+		HAS_LUCENE = False
+		
+	try:
+		import gconf
+		HAS_GCONF = True
+	except:
+		HAS_GCONF = False
+		
+	try:
+		import gnomevfs
+		HAS_GNOMEVFS = True
+	except:
+		HAS_GNOMEVFS = False
+		
+	try:
+		from xml.sax.saxutils import DefaultHandler
+		HAS_PYXML = True
+	except:
+		HAS_PYXML = False
+		
+try:
+	import pygst
+	pygst.require("0.10")
+	import gst
+	HAS_GSTREAMER = True
+except:
+	HAS_GSTREAMER = False
+	
+VERSION="2.80"
 #DEBUG
 _USE_KDE_OVERRIDE=False
+#HAS_LUCENE = False
 
 def format_size(size):
-	if size > 1000000000:
-		return "%.2f GB" % (float(size)/1000000000)
-	elif size > 1000000:
-		return str(int(round(size/1000000)))+ " MB"
-	elif size > 1000:
-		return str(int(round(size/1000)))+" KB"
+	if size > 1073741824:
+		return "%.2f GB" % (float(size)/1073741824)
+	elif size > 1048576:
+		return str(int(round(size/1048576)))+ " MB"
+	elif size > 1024:
+		return str(int(round(size/1024)))+" KB"
 	else:
 		return str(size)+" bytes"
 		
@@ -86,13 +128,16 @@ def get_play_command_for(filename):
 			print "error getting type, using kfmclient"
 			return "kfmclient exec "
 	else: #GNOME -- notice how short and sweet this is in comparison :P
-		try:
-			mimetype = gnomevfs.get_mime_type(urllib.quote(filename)) #fix for penny arcade filenames
-			full_qual_prog = gnomevfs.mime_get_default_application(mimetype)[2]
-		except:
-			print "unknown type, using gnome-open"
-			return "gnome-open "
-	
+		if HAS_GNOMEVFS:
+			try:
+				mimetype = gnomevfs.get_mime_type(urllib.quote(filename)) #fix for penny arcade filenames
+				full_qual_prog = gnomevfs.mime_get_default_application(mimetype)[2]
+			except:
+				print "unknown type, using gnome-open"
+				return "gnome-open "
+		else:
+			# :(
+			return "echo "
 	try:
 		path, program = os.path.split(full_qual_prog)
 	except:
@@ -110,29 +155,34 @@ def get_dated_dir():
 class GlobDirectoryWalker:
     # a forward iterator that traverses a directory tree
 
-    def __init__(self, directory, pattern="*"):
-        self.stack = [directory]
-        self.pattern = pattern
-        self.files = []
-        self.index = 0
+	def __init__(self, directory, pattern="*"):
+		self.stack = [directory]
+		self.pattern = pattern
+		self.files = []
+		self.index = 0
 
-    def __getitem__(self, index):
-        while 1:
-            try:
-                file = self.files[self.index]
-                self.index = self.index + 1
-            except IndexError:
-                # pop next directory from stack
-                self.directory = self.stack.pop()
-                self.files = os.listdir(self.directory)
-                self.index = 0
-            else:
-                # got a filename
-                fullname = os.path.join(self.directory, file)
-                if os.path.isdir(fullname) and not os.path.islink(fullname):
-                    self.stack.append(fullname)
-                if fnmatch.fnmatch(file, self.pattern):
-                    return fullname
+	def __getitem__(self, index):
+		while 1:
+			try:
+				file = self.files[self.index]
+				self.index = self.index + 1
+			except IndexError:
+				# pop next directory from stack
+				try:
+					while True: 
+						self.directory = self.stack.pop()
+						self.index = 0
+						self.files = os.listdir(self.directory) #loops if we have a problem listing the directory
+						break #but if it works we break
+				except OSError, e:
+					continue #evil... but it works
+			else:
+				# got a filename
+				fullname = os.path.join(self.directory, file)
+				if os.path.isdir(fullname) and not os.path.islink(fullname):
+					self.stack.append(fullname)
+				if fnmatch.fnmatch(file, self.pattern):
+					return fullname
 #usage:
 #for file in GlobDirectoryWalker(".", "*.py"):
 #    print file
@@ -179,7 +229,7 @@ def my_quote(str):
 	
 def uniquer(seq, idfun=None):
 	if not seq:
-		return None
+		return []
 	if idfun is None:
 		def idfun(x): return x
 	seen = {}
@@ -194,9 +244,9 @@ def uniquer(seq, idfun=None):
 		result.append(item)
 	return result
 	
-commands={	'play:':_("Play"),
+commands={	'play:':_("Open"),
 			'download:':_("Download"),
-			'downloadqueue:':_("Download And Play"),
+			'downloadqueue:':_("Download And Open"),
 			'pause:':_("Pause"),
 			'cancel:':_("Cancel"),
 			'file://':_("Open File"),
@@ -207,7 +257,7 @@ commands={	'play:':_("Play"),
 			'tryresume:':("Try Resume")	}
 	
 def html_command(command,arg):
-	"""returns something like '<a href="play:%s">Play</a>' for all the commands I have.
+	"""returns something like '<a href="play:%s">Open</a>' for all the commands I have.
 	Dictionary has keys of commands, and returns located strings"""
 	
 	#a couple special cases
@@ -239,19 +289,24 @@ def desktop_has_file_handler(filename):
 		# to use anyway.
 		return True
 	else:
-		# Otherwise, use GNOMEVFS to find the appropriate handler
-		handler = gnomevfs.mime_get_default_application(gnomevfs.get_mime_type(urllib.quote(filename))) #PA fix
-		if handler is not None:
+		if HAS_GNOMEVFS:
+			# Otherwise, use GNOMEVFS to find the appropriate handler
+			handler = gnomevfs.mime_get_default_application(gnomevfs.get_mime_type(urllib.quote(filename))) #PA fix
+			if handler is not None:
+				return True
+			return False
+		else: #FIXME: olpc doesn't know what the fuck... pretend yes and let error get caught later
 			return True
-		return False
 		
 def is_file_media(filename):
 	"""Returns true if this is a media file (audio or video) and false if it is any other type of file"""
 	if is_kde():
 		mime_magic = kio.KMimeMagic()
 		mimetype = str(mime_magic.findFileType(filename).mimeType())
-	else:
+	elif HAS_GNOMEVFS:
 		mimetype = gnomevfs.get_mime_type(urllib.quote(filename))
+	else:
+		return False
 	print mimetype
 	valid_mimes=['video','audio','mp4','realmedia','m4v','mov']
 	for mime in valid_mimes:
@@ -369,9 +424,6 @@ class SuperGlobal:
         if __main__.__dict__.has_key(name):
             del  __main__.__dict__[name]
             
-superglobal=SuperGlobal()
-
-
 #thanks http://www.peterbe.com/plog/html-entity-fixer
 #from htmlentitydefs import entitydefs
 #import re
@@ -391,5 +443,52 @@ def html_entity_unfixer(text):
 		text = text.replace("&"+entity+";", _my_entities[entity])
 	return text
 	
+#def lucene_escape(text):
+#	#+ - & | ! ( ) { } [ ] ^ " ~ * ? : \\
+#	escape_chars="""+-&|!(){}[]^"~*?:\\"""
+#	text = text.replace(
+
+def get_disk_free(f="/"):
+	
+	"""returns free disk space in kilobytes for 
+	   the disk that contains file f, defaulting to root.  
+	   Returns 0 on error"""
+	
+	os.stat(f) #if this fails it will raise exactly the right error
+	
+	cmd = "df -k " + f
+	
+	p = Popen(cmd, shell=True, 
+		      stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
+						 
+	lines = p.stdout.readlines()
+	
+	try:
+		return int(lines[1].split()[3]) * 1024
+	except:
+		return 0
+		
+def get_disk_total(f="/"):
+	
+	"""returns total disk space in kilobytes for 
+	   the disk that contains file f, defaulting to root.
+	   Returns 0 on error"""
+	
+	os.stat(f) #if this fails it will raise exactly the right error
+	
+	cmd = "df -k " + f
+	
+	p = Popen(cmd, shell=True, 
+		      stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
+						 
+	lines = p.stdout.readlines()
+	
+	try:
+		return int(lines[1].split()[1]) * 1024
+	except:
+		return 0
+
+
 if is_kde():
 	import kio
+
